#ifndef _Genetic_h_
#define _Genetic_h_

#ifndef _Sim_h_
# include "Sim.h"
#endif

#include <vector>
#include <string>

class Genome
{
  public:
    Genome() 
        : fitness(0) 
    {}
    
    ~Genome()
    {
        for (Exp::container::iterator i = exps.begin(); i != exps.end(); ++i)
        {
            delete (*i);
        }
        exps.clear();
    }

    static Genome *knockBoots(Genome *mom, Genome *dad);

    typedef vector<Genome *> container;
    float          fitness;
    Exp::container exps;
};

class GenePool
{
  public:
    
    GenePool();
    ~GenePool();

    int   parse(const string &filename);
    void  unparse(ostream &out) const;
    void  write(const string &filename) const;

    float getPopulation() const       { return _genomes.size(); }

    int   getMaxPopulation()          { return _maxPopulation; }
    void  setMaxPopulation(int max);

    float getMutationRate()           { return _mutationRate; }
    void  setMutationRate(float rate) { _mutationRate = rate; }

    float getReproductionRate()           { return _reproductionRate; }
    void  setReproductionRate(float rate) { _reproductionRate = rate; }

    float getSimulationLength()           { return _simulationLength; }
    void  setSimulationLength(float len)  { _simulationLength = len; }

    float getBestFitness() { return _genomes.front()->fitness; }

    void  doGeneration();

    void  deleteAll();

  private:

    Genome *          findParent();
    void              reproduce(int num);
    void              addRandomGenome(int max_depth);
    void              doMutation(float rate);
    void              modifyPopulation();
    void              simulateAll();
    void              sortByFitness();
    void              killTheWeak();

    Sim               _sim;
    Genome::container _genomes;
    Joint::container  _activeJoints;
    int               _maxPopulation;
    float             _reproductionRate;
    float             _mutationRate;
    float             _simulationLength;

};

#endif
