#include "Joint.h"

#ifndef _Math_h_
# include "Math.h"
#endif

#ifndef _Exp_h_
# include "Exp.h"
#endif

Joint::Joint(int body1, float body1off, int body2, float body2off, 
             float t_theta, Exp *t_exp)
    : b1(body1), b2(body2), b1off(body1off), b2off(body2off),
      target_theta(t_theta), target_exp(t_exp)
{
    if (b1off * b2off < 0) sign = -1.0f;
    else                   sign = 1.0f;

    //if (t_exp == 0)
    //{
    //    throw ExpExc("<Joint::Joint> target expression is null");
    //}

    while (target_theta > Math::TwoPif)
    {
        target_theta -= Math::TwoPif;
    }
    while (target_theta < -Math::TwoPif)
    {
        target_theta += Math::TwoPif;
    }

    reset();
}

Joint::~Joint()
{
    // Taken care of by the simulator
    // delete target_exp;
}

void
Joint::reset()
{
    prev_theta = target_theta;
    max_dtarget = 0.0f;
}

void
Joint::unparse(ostream &out) const
{
    out << "(joint";
    unparseRemainder(out);
}

void
Joint::unparseRemainder(ostream &out) const
{
    out << " (" << b1 << ", " << b1off << ") (" 
         << b2 << ", " << b2off << ") " << target_theta << ")";
}

//---------------------------------------------------------------------------

ConstantJoint::ConstantJoint(int body1, float body1off, 
                             int body2, float body2off, 
                             float t_theta)
    : Joint(body1, body1off, body2, body2off, t_theta, new ConstExp(0.0f))
{
    // nothing
}

ConstantJoint::~ConstantJoint()
{
    // nothing
}

void
ConstantJoint::unparse(ostream &out) const
{
    out << "(constantjoint";
    unparseRemainder(out);
}

