%{
// **********************************************************************
// Simple scanner specification.
// **********************************************************************
%}

DIGIT           [0-9]
LETTER          [a-zA-Z]
NEWLINE         [\n]
WHITESPACE      [ \t]

%{
#include <string>
#include <cstdlib>
#include "Parser.h"

static int curLine = 1; /* current line number */
static int curCol  = 1; /* current column number */

%}

%%

%{
    /* Comments --------------------------------------- */
%}

"//"[^\n]*                           { curCol += yyleng; } 

%{
    /* Bodies and joints  --------------------------------------- */
%}

"bodies"                             {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "BODIES" << endl;
                                       return(BODIES);
                                     }

"body"                               {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "BODY" << endl;
                                       return(BODY);
                                     }

"joints"                             {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "JOINTS" << endl;
                                       return(JOINTS);
                                     }

"joint"                              {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "JOINT" << endl;
                                       return(JOINT);
                                     }

"constantjoint"                      {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "CONSTANTJOINT" << endl;
                                       return(CONSTANTJOINT);
                                     }

"exps"                               {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "EXPS" << endl;
                                       return(EXPS);
                                     }


%{
    /* Exp names --------------------------------------- */
%}

"sin"                                {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "SIN" << endl;
                                       return(SIN);
                                     }
"cos"                                {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "COS" << endl;
                                       return(COS);
                                     }
"t"                                  {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "TIME" << endl;
                                       return(TIME);
                                     }
"+"                                  {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "PLUS" << endl;
                                       return(PLUS);
                                     }
"-"                                  {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "UNARYMINUS" << endl;
                                       return(UNARYMINUS);
                                     }
"*"                                  {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "TIMES" << endl;
                                       return(TIMES);
                                     }
">"                                  {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "GREATER" << endl;
                                       return(GREATER);
                                     }
"<"                                  {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "LESS" << endl;
                                       return(LESS);
                                     }
-?{DIGIT}+(.{DIGIT}+)?(e-?{DIGIT}+)? {
                                       yylval.TOKENval.value = atof(yytext);
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "CONSTANT: " 
                                       //     << yylval.TOKENval.value << endl;
                                       return(CONSTANT);
                                     }

%{
    /* Other patterns --------------------------------------- */
%}

"("                                  {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "LPAREN" << endl;
                                       return(LPAREN);
                                     }
")"                                  {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "RPAREN" << endl;
                                       return(RPAREN);
                                     }
","                                  {
                                       yylval.TOKENval.lineNum = curLine;
                                       yylval.TOKENval.colNum = curCol;
                                       curCol += yyleng;
                                       //cout << "COMMA" << endl;
                                       return(COMMA);
                                     }
%{
    /* Integers --------------------------------------- */
%}

%{
    /* Whitespace --------------------------------------- */
%}

{NEWLINE}+                           { 
                                       curLine += yyleng;
                                       curCol = 1;
                                     }
{WHITESPACE}+                        { curCol += yyleng; }

%{
    /* Default --------------------------------------- */
%}

.                                    { 
                                       char msg[256];
                                       sprintf(msg, "<scanner> line %d, col "
                                               "%d: ignoring illegal character",
                                               curLine, curCol);
                                       yyerror(msg);
                                       curCol++;
                                     }
